# Copyright (C) 2010 by Dr. Dieter Maurer <dieter@handshake.de>; see 'LICENSE.txt' for details
"""Binding support."""

SoapBinding = "urn:oasis:names:tc:SAML:2.0:bindings:SOAP"
HttpRedirectBinding = "urn:oasis:names:tc:SAML:2.0:bindings:HTTP-Redirect"
HttpPostBinding = "urn:oasis:names:tc:SAML:2.0:bindings:HTTP-POST"
